#!/usr/bin/perl -w
#
# dobuild.pl - perl script for building any project
#
# edmund@roland.org
#
# THIS VERSION IS FOR LINUX ONLY!!!
#

use File::Path;
use File::stat;

#
# build settings
#
my $root = "/epoc";

#
# projects
#
my @projects=("e32frodo", "esidplay", "snus", "vim", "drep", "hexdump"); # <=== default
#my @projects=("hexdump");

#
# platforms
#
my @platforms=("armi");
#my @platforms=("armi");

#
# SDKs
#
#my @sdks=("s60cpp02", "n9210cpp11", "quartz"); # <=== default
my @sdks=("n9210cpp11");
#my @sdks=("s60cpp02");

my $outdir="/var/www/html/build/";
my $logdir="$outdir/logs/";
my $sourcedir = "$root/daily_builds";
my $cvslog = $logdir . "cvs.log";
my $summary = $logdir . "summary.log";

unlink $summary;

($tmp,$tmp,$tmp,$mday, $mon, $year) = localtime;
$year += 1900;
$mon += 1;
my $date = $year . "." . $mon . "." . $mday;

#
# directory check
#
mkdir $logdir;
foreach (@sdks) {
	my $dir = "$root/$_";	
	(-d $dir) or die "$dir does not exist, $!";
}


#
# checkout source from CVS
#
unlink $cvslog;
print "checking out from CVS... \n";
chdir($sourcedir) or warn("could not walk to $sourcedir ($!)");
foreach (@projects) {
  print "module: $_\n";
#  system("cvs -d:pserver:anonymous\@cvs.$_.sourceforge.net:/cvsroot/$_ login >> ${cvslog} 2>&1");

  # cheap tricks :(
  if(/^vim|drep|hexdump$/) {
    print "(skipping $_)\n";
    next;
  } elsif(/^snus$/) {
    system("cvs -z3 -d:pserver:anonymous\@cvs.gnupoc.sourceforge.net:/cvsroot/gnupoc co $_ >> ${cvslog} 2>&1");
  } else {
    system("cvs -z3 -d:pserver:anonymous\@cvs.$_.sourceforge.net:/cvsroot/$_ co $_ >> ${cvslog} 2>&1");
  }
}


# add path to GCC cross compiler for ARM
my $path = $ENV{PATH};
$path = "/usr/local/er6/bin:" . $path;
$ENV{PATH} = $path;

print "environment variables:\n";
print "\n";
print "    EPOCROOT      = $ENV{EPOCROOT}\n";
print "    GNUPOC_PETRAN = $ENV{GNUPOC_PETRAN}\n";
print "    PATH          = $ENV{PATH}\n";
print "\n";

#
# Build the source and save it somewhere
#
print "starting build process... \n";
print "\n";
print "  projects:   "; foreach (@projects) {print "$_ ";} print "\n";
print "  platforms:  "; foreach (@platforms) {print "$_ ";} print "\n";
print "  sdks:       "; foreach (@sdks) {print "$_ ";} print "\n";
print "  date:       $date\n";
print "\n";

my $tick = time;
print "Building...(this might take some time. Get a cup of Coffee....)\n";

my $sanitylog = $logdir . "sanitylog.log";
system("rm -f $sanitylog");
system("echo 'Sanity log - should be empty if OK' > $sanitylog");

open FO, ">$summary" or die "Could not create $summary";

foreach $sdk (@sdks) {
  print "sdk: $sdk\n";
  system("echo SDK: $sdk >> $sanitylog");

  my $epocroot = "$root/$sdk/";
  $ENV{EPOCROOT} = $epocroot;
  my $path = $ENV{PATH};
  $path = "$epocroot/epoc32/tools:" . $path;
  $ENV{PATH} = $path;

  foreach $plat (@platforms) {
    print " plat: $plat\n";
    system("echo  PLAT: $plat >> $sanitylog");

    foreach $proj (@projects) {
      print "  proj: $proj\n";
      system("echo   PROJ: $proj >> $sanitylog");

      my $logfile = $logdir . "log_" . $proj . "_" . $plat . "_" . $sdk . ".log";
      unlink $logfile;

      my $dir = "$sourcedir/$proj/group";
      chdir($dir);

      system("bldmake -v bldfiles >> ${logfile} 2>&1") && print "bldmake bldfiles failed, $!";
      #
      # core
      #
      system("abld -v export >> ${logfile} 2>&1") && print "abld export $plat failed, $!";
      system("abld -v -k makefile $plat >> ${logfile} 2>&1") && print "abld makefile $plat failed, $!";
      system("abld -v -k library $plat >> ${logfile} 2>&1") && print "abld library $plat failed, $!";
      system("abld -v -k target $plat >> ${logfile} 2>&1") && print "abld target $plat failed, $!";

      # sanity check
      system("abld -c build $plat >> $sanitylog 2>&1") && print "abld -c $plat failed, $!";
      #
      # test
      #
#      system("abld -v -k test build $plat >> ${logfile} 2>&1") && die "abld test build $plat failed, $!";

      print FO "Project=$proj Platform=$plat SDK=$sdk:\n";
      print FO "\n";
      my $warnings = `grep -i "warning:" ${logfile} | wc -l`;
      my $errors   = `grep 'Error ' ${logfile} | wc -l`;
      chomp $warnings;
      chomp $errors;
      print FO "  warnings: $warnings\n";
      print FO "  errors:   $errors\n";
      print FO "\n";

    }

  }

}

close FO;

#
# create source tarball
#
mkdir "$outdir/src";
foreach $proj (@projects) {
  my $tarball = "$outdir/src/$proj.src.$date.tar.gz";
  my $logtar = $logdir . "tarlog_" . $proj . ".log";
  unlink $logtar;
  print "creating source tarball for '$proj' ($tarball)\n";
  chdir($sourcedir);
  unlink $tarball or warn("could not unlink ($!)");
  system("tar -hczvf $tarball $proj/ > $logtar 2>&1") && warn("hmm. could not execute tar, $!")
}


#
# collect some info
#
my $tock = time;
my $buildtime = $tock - $tick;
($sec, $min, $hour) = localtime($buildtime);
$buildtime = "$hour hours, $min minutes, $sec seconds";

my $warnings = `grep -ir "warning:" ${logdir} | wc -l`;
my $errors   = `grep -r 'Error ' ${logdir} | wc -l`;

# some status of the build
chdir($root);


#
# generate HTML output
#
print "Generating HTML output\n";
my $report = "$outdir/" . "index.html";
open FO, ">$report" or die "Could not create $report";
print FO "<html>\n";
print FO "<head>\n";
print FO "  <title>Daily EPOC builds</title>\n";
print FO "</head>\n";

print FO "</body>\n";
print FO "<center><b>Daily EPOC builds</b></center>\n";
print FO "<hr>\n";
print FO "<p>\n";

print FO "<table cellpadding=3 border=\"1\">\n";
print FO "<tr><td>date</td>         <td>$date</td></tr>\n";
print FO "<tr><td>buildtime</td>    <td>$buildtime</td></tr>\n";
print FO "<tr><td>warnings</td>     <td>$warnings</td></tr>\n";
print FO "<tr><td>errors</td>       <td>$errors</td></tr>\n";
print FO "</table>\n";
print FO "<br>\n";

#
# projects
#
print FO "<b>Projects:</b><br>\n";
print FO "<ul>\n";
foreach (@projects) {
  my $tarball = "$outdir/src/$_.src.$date.tar.gz";
  my $size = -1;
  if(-e $tarball) {
    my $sb = stat($tarball);
    $size = $sb->size;
  }
  my $lines = `wc -l \`find $sourcedir/$_\` | grep total`;
  $lines =~ m/(.*) total/;
  print FO "<li>$_<a href=\"src/$_.src.$date.tar.gz\">(source)</a> ($size bytes, $1 lines of code)<br>";
}
print FO "</ul>\n";


print FO "<b>Platforms:</b><br>\n";
print FO "<ul>\n";
foreach (@platforms) {
  print FO "<li>$_<br>";
}
print FO "</ul>\n";

print FO "<b>SDKs:</b><br>\n";
print FO "<ul>\n";
foreach (@sdks) {
  print FO "<li>$_<br>";
}
print FO "</ul>\n";

#
# Logs
#
print FO "<b>Logs:</b><br>\n";
print FO "<ul>\n";
print FO "<li><a href=\"logs/cvs.log\"><b>CVS log</b></a><br>";
print FO "<li><a href=\"logs/sanitylog.log\"><b>Sanity log</b></a><br>";
print FO "<li><a href=\"logs/main.log\"><b>Main build log</b></a><br>";
print FO "<li><a href=\"logs/summary.log\"><b>Summary log</b></a><br>";
print FO "<li><a href=\"logs/\"><b>Log directory</b></a><br>";
print FO "</ul>\n";

system("cp `which dobuild.pl` $outdir/.");
$dobuild = "$outdir/dobuild.pl";
print FO "Download this perl script here: <a href=\"${dobuild}\">dobuild.pl </a>";

print FO "<hr>\n";
$me = `whoami`;
$where = `hostname`;
print FO "$me\@$where<br>\n";

print FO "</body>\n";
print FO "</html>\n";
close FO;
