#! /usr/bin/perl -w
#
# sdk-unpack.pl - Rich Williams <rdw@perldog.com>
#
# Part of GnuPoc project - EPOC SDK for GNU/Linux http://gnupoc.sourceforge.net/
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License version 2 as
#    published by the Free Software Foundation.
#

use strict;

############################################################################

my $WINE = "/usr/bin/wine";
my $I6C = "/opt/wine/exe/i6comp.exe";
my $CAB = "/cdrom1/data1.cab";
my $DIR = "/nokia/install";

############################################################################

sub get_components {

  my $result;
  open LISTCOMP, "$WINE $I6C c $CAB|" or die;
  local $/ = "\r\n";
  while (<LISTCOMP>) {
    last if /^=+/;
  }
  local $/ = "\r\n\r\n";
  while (<LISTCOMP>) {
    last if /Wine exited/;
    tr/\r//d;
    my $component;
    foreach (split /\n/) {
      my($key, $value) = split /\s+:\s+/, $_;
      next unless $value;
      $value = [split /,\s+/, $value] if $key =~ /(SubComps|File Groups)/;
      $component->{$key} = $value;
    }
    $result->{$component->{'Comp Identity'}} = $component;
  }
  close LISTCOMP;
  return $result;
}

############################################################################

sub install_filegroup {

  my($dir, $group) = @_;

  chdir($dir) or die "Failed to chdir ($dir) - $!\n";
  system("$WINE -- $I6C x '-g$group' $CAB >/dev/null");
}

############################################################################

sub install_component {

  my($comps, $basedir, $base, $name) = @_;

  return if $name =~ /Disk</;
  return unless ((my $dir = $name) =~ s/<$base>/$basedir/);

  $dir =~ tr/\\/\//;

  my $desc = $comps->{$name}->{Description};
  printf "Installing $name %s\n", $desc ? "($desc)" : '';

  unless (-d $dir) {
    mkdir $dir or die "Failed to mkdir ($dir) - $!\n";
  }
  
  foreach (@{$comps->{$name}->{'SubComps'}}) {
    install_component($comps, $basedir, $base, $_);
  }

  foreach (@{$comps->{$name}->{'File Groups'}}) {
    install_filegroup($dir, $_);
  }
}

############################################################################

install_component(get_components(), $DIR, 'Data', '<Data>');
